@@ -340,9 +340,9 @@ load_ontology_from_journal (void)
 			gint subject_id, predicate_id, object_id;
 
 			if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
-				tracker_db_journal_reader_get_statement (&subject_id, &predicate_id, &object);
+				tracker_db_journal_reader_get_statement (NULL, &subject_id, &predicate_id, &object);
 			} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT_ID) {
-				tracker_db_journal_reader_get_statement_id (&subject_id, &predicate_id, &object_id);
+				tracker_db_journal_reader_get_statement_id (NULL, &subject_id, &predicate_id, &object_id);
 				object = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (object_id));
 			} else {
 				continue;
@@ -411,8 +411,8 @@ replay_journal (void)
 	while (tracker_db_journal_reader_next (&journal_error)) {
 		GError *error = NULL;
 		TrackerDBJournalEntryType type;
-		const gchar *subject, *predicate, *object;
-		gint subject_id, predicate_id, object_id;
+		const gchar *graph, *subject, *predicate, *object;
+		gint graph_id, subject_id, predicate_id, object_id;
 
 		type = tracker_db_journal_reader_get_type ();
 		if (type == TRACKER_DB_JOURNAL_RESOURCE) {
@@ -438,35 +438,55 @@ replay_journal (void)
 		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
 			tracker_data_commit_transaction ();
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
-			tracker_db_journal_reader_get_statement (&subject_id, &predicate_id, &object);
+			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
 
+			if (graph_id > 0) {
+				graph = query_resource_by_id (graph_id);
+			} else {
+				graph = NULL;
+			}
 			subject = query_resource_by_id (subject_id);
 			predicate = query_resource_by_id (predicate_id);
 
-			tracker_data_insert_statement_with_string (NULL, subject, predicate, object, &error);
+			tracker_data_insert_statement_with_string (graph, subject, predicate, object, &error);
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT_ID) {
-			tracker_db_journal_reader_get_statement_id (&subject_id, &predicate_id, &object_id);
+			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
 
+			if (graph_id > 0) {
+				graph = query_resource_by_id (graph_id);
+			} else {
+				graph = NULL;
+			}
 			subject = query_resource_by_id (subject_id);
 			predicate = query_resource_by_id (predicate_id);
 			object = query_resource_by_id (object_id);
 
-			tracker_data_insert_statement_with_uri (NULL, subject, predicate, object, &error);
+			tracker_data_insert_statement_with_uri (graph, subject, predicate, object, &error);
 		} else if (type == TRACKER_DB_JOURNAL_DELETE_STATEMENT) {
-			tracker_db_journal_reader_get_statement (&subject_id, &predicate_id, &object);
+			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
 
+			if (graph_id > 0) {
+				graph = query_resource_by_id (graph_id);
+			} else {
+				graph = NULL;
+			}
 			subject = query_resource_by_id (subject_id);
 			predicate = query_resource_by_id (predicate_id);
 
-			tracker_data_delete_statement (NULL, subject, predicate, object, &error);
+			tracker_data_delete_statement (graph, subject, predicate, object, &error);
 		} else if (type == TRACKER_DB_JOURNAL_DELETE_STATEMENT_ID) {
-			tracker_db_journal_reader_get_statement_id (&subject_id, &predicate_id, &object_id);
+			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
 
+			if (graph_id > 0) {
+				graph = query_resource_by_id (graph_id);
+			} else {
+				graph = NULL;
+			}
 			subject = query_resource_by_id (subject_id);
 			predicate = query_resource_by_id (predicate_id);
 			object = query_resource_by_id (object_id);
 
-			tracker_data_delete_statement (NULL, subject, predicate, object, &error);
+			tracker_data_delete_statement (graph, subject, predicate, object, &error);
 		}
 	}
 
@@ -805,20 +825,38 @@ create_decomposed_metadata_property_table (TrackerDBInterface *iface,
 	}
 
 	if (transient || tracker_property_get_multiple_values (property)) {
+		GString *sql;
+
+		sql = g_string_new ("");
+		g_string_append_printf (sql, "CREATE %sTABLE \"%s_%s\" ("
+		                             "ID INTEGER NOT NULL, "
+		                             "\"%s\" %s NOT NULL, "
+		                             "\"%s:graph\" INTEGER",
+		                             transient ? "TEMPORARY " : "",
+		                             service_name,
+		                             field_name,
+		                             field_name,
+		                             sql_type,
+		                             field_name);
+
+		if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME) {
+			/* xsd:dateTime is stored in three columns:
+			 * universal time, local date, local time of day */
+			g_string_append_printf (sql,
+			                        ", \"%s:localDate\" INTEGER NOT NULL"
+			                        ", \"%s:localTime\" INTEGER NOT NULL",
+			                        tracker_property_get_name (property),
+			                        tracker_property_get_name (property));
+		}
+
 		/* multiple values */
 		if (tracker_property_get_indexed (property)) {
 			/* use different UNIQUE index for properties whose
 			 * value should be indexed to minimize index size */
 			tracker_db_interface_execute_query (iface, NULL,
-			                                    "CREATE %sTABLE \"%s_%s\" ("
-			                                    "ID INTEGER NOT NULL, "
-			                                    "\"%s\" %s NOT NULL, "
+			                                    "%s, "
 			                                    "UNIQUE (\"%s\", ID))",
-			                                    transient ? "TEMPORARY " : "",
-			                                    service_name,
-			                                    field_name,
-			                                    field_name,
-			                                    sql_type,
+			                                    sql->str,
 			                                    field_name);
 
 			tracker_db_interface_execute_query (iface, NULL,
@@ -831,17 +869,13 @@ create_decomposed_metadata_property_table (TrackerDBInterface *iface,
 			/* we still have to include the property value in
 			 * the unique index for proper constraints */
 			tracker_db_interface_execute_query (iface, NULL,
-			                                    "CREATE %sTABLE \"%s_%s\" ("
-			                                    "ID INTEGER NOT NULL, "
-			                                    "\"%s\" %s NOT NULL, "
+			                                    "%s, "
 			                                    "UNIQUE (ID, \"%s\"))",
-			                                    transient ? "TEMPORARY " : "",
-			                                    service_name,
-			                                    field_name,
-			                                    field_name,
-			                                    sql_type,
+			                                    sql->str,
 			                                    field_name);
 		}
+
+		g_string_free (sql, TRUE);
 	} else if (sql_type_for_single_value) {
 		*sql_type_for_single_value = sql_type;
 	}
@@ -898,6 +932,17 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 				if (tracker_property_get_is_inverse_functional_property (property)) {
 					g_string_append (sql, " UNIQUE");
 				}
+
+				g_string_append_printf (sql, ", \"%s:graph\" INTEGER",
+				                        tracker_property_get_name (property));
+
+				if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME) {
+					/* xsd:dateTime is stored in three columns:
+					 * universal time, local date, local time of day */
+					g_string_append_printf (sql, ", \"%s:localDate\" INTEGER, \"%s:localTime\" INTEGER",
+						                tracker_property_get_name (property),
+						                tracker_property_get_name (property));
+				}
 			}
 		}
 	}
