@@ -25,7 +25,7 @@
 #include <time.h>
 
 #include <libtracker-common/tracker-common.h>
-#include <libtracker-common/tracker-type-utils.h>
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-ontology.h>
 
@@ -79,7 +79,9 @@ struct _TrackerDataUpdateBufferResource {
 struct _TrackerDataUpdateBufferProperty {
 	gchar *name;
 	GValue value;
-	gboolean fts;
+	gchar *graph;
+	gboolean fts : 1;
+	gboolean date_time : 1;
 };
 
 struct _TrackerDataUpdateBufferTable {
@@ -276,6 +278,7 @@ cache_table_free (TrackerDataUpdateBufferTable *table)
 		property = &g_array_index (table->properties, TrackerDataUpdateBufferProperty, i);
 		g_free (property->name);
 		g_value_unset (&property->value);
+		g_free (property->graph);
 	}
 
 	g_array_free (table->properties, TRUE);
@@ -312,15 +315,19 @@ static void
 cache_insert_value (const gchar            *table_name,
                     const gchar            *field_name,
                     GValue                 *value,
+                    const gchar            *graph,
                     gboolean                multiple_values,
-                    gboolean                fts)
+                    gboolean                fts,
+                    gboolean                date_time)
 {
 	TrackerDataUpdateBufferTable    *table;
 	TrackerDataUpdateBufferProperty  property;
 
 	property.name = g_strdup (field_name);
 	property.value = *value;
+	property.graph = g_strdup (graph);
 	property.fts = fts;
+	property.date_time = date_time;
 
 	table = cache_ensure_table (table_name, multiple_values);
 	g_array_append_val (table->properties, property);
@@ -341,14 +348,17 @@ cache_delete_value (const gchar            *table_name,
                     const gchar            *field_name,
                     GValue                 *value,
                     gboolean                multiple_values,
-                    gboolean                fts)
+                    gboolean                fts,
+                    gboolean                date_time)
 {
 	TrackerDataUpdateBufferTable    *table;
 	TrackerDataUpdateBufferProperty  property;
 
 	property.name = g_strdup (field_name);
 	property.value = *value;
+	property.graph = NULL;
 	property.fts = fts;
+	property.date_time = date_time;
 
 	table = cache_ensure_table (table_name, multiple_values);
 	table->delete_value = TRUE;
@@ -412,24 +422,33 @@ ensure_resource_id (const gchar *uri,
 
 static void
 statement_bind_gvalue (TrackerDBStatement *stmt,
-                       gint                idx,
+                       gint               *idx,
                        const GValue       *value)
 {
-	switch (G_VALUE_TYPE (value)) {
+	GType type;
+
+	type = G_VALUE_TYPE (value);
+	switch (type) {
 	case G_TYPE_STRING:
-		tracker_db_statement_bind_text (stmt, idx, g_value_get_string (value));
+		tracker_db_statement_bind_text (stmt, (*idx)++, g_value_get_string (value));
 		break;
 	case G_TYPE_INT:
-		tracker_db_statement_bind_int (stmt, idx, g_value_get_int (value));
+		tracker_db_statement_bind_int (stmt, (*idx)++, g_value_get_int (value));
 		break;
 	case G_TYPE_INT64:
-		tracker_db_statement_bind_int64 (stmt, idx, g_value_get_int64 (value));
+		tracker_db_statement_bind_int64 (stmt, (*idx)++, g_value_get_int64 (value));
 		break;
 	case G_TYPE_DOUBLE:
-		tracker_db_statement_bind_double (stmt, idx, g_value_get_double (value));
+		tracker_db_statement_bind_double (stmt, (*idx)++, g_value_get_double (value));
 		break;
 	default:
-		g_warning ("Unknown type for binding: %s\n", G_VALUE_TYPE_NAME (value));
+		if (type == TRACKER_TYPE_DATE_TIME) {
+			tracker_db_statement_bind_int64 (stmt, (*idx)++, tracker_date_time_get_time (value));
+			tracker_db_statement_bind_int (stmt, (*idx)++, tracker_date_time_get_local_date (value));
+			tracker_db_statement_bind_int (stmt, (*idx)++, tracker_date_time_get_local_time (value));
+		} else {
+			g_warning ("Unknown type for binding: %s\n", G_VALUE_TYPE_NAME (value));
+		}
 		break;
 	}
 }
@@ -444,7 +463,7 @@ tracker_data_resource_buffer_flush (GError **error)
 	GHashTableIter                  iter;
 	const gchar                    *table_name;
 	GString                        *sql, *fts;
-	int                             i;
+	gint                            i, param;
 	GError                         *actual_error = NULL;
 
 	iface = tracker_db_manager_get_db_interface ();
@@ -479,15 +498,33 @@ tracker_data_resource_buffer_flush (GError **error)
 					                                              "DELETE FROM \"%s\" WHERE ID = ? AND \"%s\" = ?",
 					                                              table_name,
 					                                              property->name);
+				} else if (property->date_time) {
+					stmt = tracker_db_interface_create_statement (iface,
+					                                              "INSERT OR IGNORE INTO \"%s\" (ID, \"%s\", \"%s:localDate\", \"%s:localTime\", \"%s:graph\") VALUES (?, ?, ?, ?, ?)",
+					                                              table_name,
+					                                              property->name,
+					                                              property->name,
+					                                              property->name,
+					                                              property->name);
 				} else {
 					stmt = tracker_db_interface_create_statement (iface,
-					                                              "INSERT OR IGNORE INTO \"%s\" (ID, \"%s\") VALUES (?, ?)",
+					                                              "INSERT OR IGNORE INTO \"%s\" (ID, \"%s\", \"%s:graph\") VALUES (?, ?, ?)",
 					                                              table_name,
+					                                              property->name,
 					                                              property->name);
 				}
 
-				tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
-				statement_bind_gvalue (stmt, 1, &property->value);
+				param = 0;
+
+				tracker_db_statement_bind_int (stmt, param++, resource_buffer->id);
+				statement_bind_gvalue (stmt, &param, &property->value);
+
+				if (property->graph) {
+					tracker_db_statement_bind_int (stmt, param++, ensure_resource_id (property->graph, NULL));
+				} else {
+					tracker_db_statement_bind_null (stmt, param++);
+				}
+
 				tracker_db_statement_execute (stmt, &actual_error);
 				g_object_unref (stmt);
 
@@ -569,23 +606,37 @@ tracker_data_resource_buffer_flush (GError **error)
 					g_string_append (sql, ", ");
 				}
 				g_string_append_printf (sql, "\"%s\" = ?", property->name);
+
+				if (property->date_time) {
+					g_string_append_printf (sql, ", \"%s:localDate\" = ?", property->name);
+					g_string_append_printf (sql, ", \"%s:localTime\" = ?", property->name);
+				}
+
+				g_string_append_printf (sql, ", \"%s:graph\" = ?", property->name);
 			}
 
 			g_string_append (sql, " WHERE ID = ?");
 
 			stmt = tracker_db_interface_create_statement (iface, "%s", sql->str);
-			tracker_db_statement_bind_int (stmt, i, resource_buffer->id);
 
+			param = 0;
 			for (i = 0; i < table->properties->len; i++) {
 				property = &g_array_index (table->properties, TrackerDataUpdateBufferProperty, i);
 				if (table->delete_value) {
 					/* just set value to NULL for single value properties */
-					tracker_db_statement_bind_null (stmt, i);
+					tracker_db_statement_bind_null (stmt, param++);
 				} else {
-					statement_bind_gvalue (stmt, i, &property->value);
+					statement_bind_gvalue (stmt, &param, &property->value);
+				}
+				if (property->graph) {
+					tracker_db_statement_bind_int (stmt, param++, ensure_resource_id (property->graph, NULL));
+				} else {
+					tracker_db_statement_bind_null (stmt, param++);
 				}
 			}
 
+			tracker_db_statement_bind_int (stmt, param++, resource_buffer->id);
+
 			tracker_db_statement_execute (stmt, &actual_error);
 			g_object_unref (stmt);
 
@@ -765,7 +816,7 @@ cache_create_service_decomposed (TrackerClass *cl,
 	cache_insert_row (cl);
 
 	g_value_set_int (&gvalue, ensure_resource_id (tracker_class_get_uri (cl), NULL));
-	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue, TRUE, FALSE);
+	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue, graph, TRUE, FALSE, FALSE);
 
 	tracker_class_set_count (cl, tracker_class_get_count (cl) + 1);
 	if (insert_callbacks) {
@@ -802,6 +853,12 @@ value_equal (GValue *value1,
 		/* does RDF define equality for floating point values? */
 		return g_value_get_double (value1) == g_value_get_double (value2);
 	default:
+		if (type == TRACKER_TYPE_DATE_TIME) {
+			/* ignore UTC offset for comparison, irrelevant for comparison according to xsd:dateTime spec
+			 * http://www.w3.org/TR/xmlschema-2/#dateTime
+			 */
+			return tracker_date_time_get_time (value1) == tracker_date_time_get_time (value2);
+		}
 		g_assert_not_reached ();
 	}
 }
@@ -1015,8 +1072,8 @@ string_to_gvalue (const gchar         *value,
 		break;
 	case TRACKER_PROPERTY_TYPE_DATE:
 	case TRACKER_PROPERTY_TYPE_DATETIME:
-		g_value_init (gvalue, G_TYPE_INT);
-		g_value_set_int (gvalue, tracker_string_to_date (value));
+		g_value_init (gvalue, TRACKER_TYPE_DATE_TIME);
+		tracker_date_time_set_from_string (gvalue, value);
 		break;
 	case TRACKER_PROPERTY_TYPE_RESOURCE:
 		object_id = ensure_resource_id (value, NULL);
@@ -1032,6 +1089,7 @@ string_to_gvalue (const gchar         *value,
 static void
 cache_set_metadata_decomposed (TrackerProperty  *property,
                                const gchar      *value,
+                               const gchar      *graph,
                                GError          **error)
 {
 	gboolean            multiple_values, fts;
@@ -1044,7 +1102,7 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 	/* also insert super property values */
 	super_properties = tracker_property_get_super_properties (property);
 	while (*super_properties) {
-		cache_set_metadata_decomposed (*super_properties, value, error);
+		cache_set_metadata_decomposed (*super_properties, value, graph, error);
 		if (*error) {
 			return;
 		}
@@ -1085,7 +1143,8 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		             resource_buffer->subject,
 		             field_name);
 	} else {
-		cache_insert_value (table_name, field_name, &gvalue, multiple_values, fts);
+		cache_insert_value (table_name, field_name, &gvalue, graph, multiple_values, fts,
+		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 	}
 
 	g_free (table_name);
@@ -1130,7 +1189,8 @@ delete_metadata_decomposed (TrackerProperty  *property,
 		/* value not found */
 		g_value_unset (&gvalue);
 	} else {
-		cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts);
+		cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts,
+		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 	}
 
 	g_free (table_name);
@@ -1232,7 +1292,8 @@ cache_delete_resource_type (TrackerClass *class,
 			g_value_copy (old_gvalue, &gvalue);
 
 			value_set_remove_value (old_values, &gvalue);
-			cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts);
+			cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts,
+			                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
 		}
 
 		g_free (table_name);
@@ -1299,7 +1360,9 @@ tracker_data_delete_statement (const gchar  *graph,
 		class = tracker_ontology_get_class_by_uri (object);
 		if (class != NULL) {
 			if (!in_journal_replay) {
-				tracker_db_journal_append_delete_statement_id (resource_buffer->id,
+				tracker_db_journal_append_delete_statement_id (
+					(graph != NULL ? query_resource_id (graph) : 0),
+					resource_buffer->id,
 					tracker_data_query_resource_id (predicate),
 					query_resource_id (object));
 			}
@@ -1315,11 +1378,15 @@ tracker_data_delete_statement (const gchar  *graph,
 			gint id = tracker_property_get_id (field);
 			if (!in_journal_replay) {
 				if (tracker_property_get_data_type (field) == TRACKER_PROPERTY_TYPE_RESOURCE) {
-					tracker_db_journal_append_delete_statement_id (resource_buffer->id,
+					tracker_db_journal_append_delete_statement_id (
+						(graph != NULL ? query_resource_id (graph) : 0),
+						resource_buffer->id,
 						(id != 0) ? id : tracker_data_query_resource_id (predicate),
 						query_resource_id (object));
 				} else {
-					tracker_db_journal_append_delete_statement (resource_buffer->id,
+					tracker_db_journal_append_delete_statement (
+						(graph != NULL ? query_resource_id (graph) : 0),
+						resource_buffer->id,
 						(id != 0) ? id : tracker_data_query_resource_id (predicate),
 						object);
 				}
@@ -1415,7 +1482,7 @@ tracker_data_insert_statement_common (const gchar            *graph,
 		                                                 g_free, (GDestroyNotify) cache_table_free);
 
 		g_value_set_int (&gvalue, tracker_data_update_get_next_modseq ());
-		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue, FALSE, FALSE);
+		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue, graph, FALSE, FALSE, FALSE);
 
 		g_hash_table_insert (update_buffer.resources, g_strdup (subject), resource_buffer);
 	}
@@ -1548,7 +1615,7 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		resource_buffer->new_subject = g_strdup (object);
 	} else {
 		/* add value to metadata database */
-		cache_set_metadata_decomposed (property, object, &actual_error);
+		cache_set_metadata_decomposed (property, object, graph, &actual_error);
 		if (actual_error) {
 			tracker_data_update_buffer_clear ();
 			g_propagate_error (error, actual_error);
@@ -1569,7 +1636,9 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 	}
 
 	if (!in_journal_replay) {
-		tracker_db_journal_append_insert_statement_id (resource_buffer->id,
+		tracker_db_journal_append_insert_statement_id (
+			(graph != NULL ? query_resource_id (graph) : 0),
+			resource_buffer->id,
 			(prop_id != 0) ? prop_id : tracker_data_query_resource_id (predicate),
 			query_resource_id (object));
 	}
@@ -1615,7 +1684,7 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 	}
 
 	/* add value to metadata database */
-	cache_set_metadata_decomposed (property, object, &actual_error);
+	cache_set_metadata_decomposed (property, object, graph, &actual_error);
 	if (actual_error) {
 		tracker_data_update_buffer_clear ();
 		g_propagate_error (error, actual_error);
@@ -1635,7 +1704,9 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 	}
 
 	if (!in_journal_replay) {
-		tracker_db_journal_append_insert_statement (resource_buffer->id,
+		tracker_db_journal_append_insert_statement (
+			(graph != NULL ? query_resource_id (graph) : 0),
+			resource_buffer->id,
 			(id != 0) ? id : tracker_data_query_resource_id (predicate),
 			object);
 	}
@@ -1960,6 +2031,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 	TrackerDBStatement *stmt;
 	TrackerDBResultSet *result_set, *single_result, *multi_result;
 	TrackerClass       *class;
+	gchar              *urn;
 	GString            *sql;
 	TrackerProperty   **properties, *property;
 	int                 i;
@@ -1975,13 +2047,14 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 	/* DROP GRAPH <url> - url here is nie:url */
 
-	stmt = tracker_db_interface_create_statement (iface, "SELECT ID FROM \"nie:DataObject\" WHERE \"nie:DataObject\".\"nie:url\" = ?");
+	stmt = tracker_db_interface_create_statement (iface, "SELECT ID, (SELECT Uri FROM Resource WHERE ID = \"nie:DataObject\".ID) FROM \"nie:DataObject\" WHERE \"nie:DataObject\".\"nie:url\" = ?");
 	tracker_db_statement_bind_text (stmt, 0, url);
 	result_set = tracker_db_statement_execute (stmt, NULL);
 	g_object_unref (stmt);
 
 	if (result_set) {
 		tracker_db_result_set_get (result_set, 0, &resource_id, -1);
+		tracker_db_result_set_get (result_set, 1, &urn, -1);
 		g_object_unref (result_set);
 	} else {
 		/* For fallback to the old behaviour, we could do this here:
@@ -2073,7 +2146,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 					tracker_db_result_set_get (single_result, i++, &value, -1);
 
 					if (value) {
-						tracker_data_delete_statement (graph, url,
+						tracker_data_delete_statement (graph, urn,
 						                               tracker_property_get_uri (property),
 						                               value,
 						                               &new_error);
@@ -2108,7 +2181,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 							tracker_db_result_set_get (multi_result, 0, &value, -1);
 
-							tracker_data_delete_statement (graph, url,
+							tracker_data_delete_statement (graph, urn,
 							                               tracker_property_get_uri (property),
 							                               value,
 							                               &new_error);
@@ -2138,6 +2211,8 @@ tracker_data_delete_resource_description (const gchar *graph,
 
 		g_object_unref (result_set);
 	}
+
+	g_free (urn);
 }
 
 
