@@ -33,10 +33,10 @@
 
 #include <glib/gstdio.h>
 
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-ontology.h>
 #include <libtracker-common/tracker-property.h>
-#include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
 #include "tracker-db-journal.h"
@@ -54,7 +54,7 @@
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_10
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_11
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 
 #define IN_USE_FILENAME               ".meta.isrunning"
@@ -76,7 +76,8 @@ typedef enum {
 	TRACKER_DB_VERSION_07,      /* Version 0.7.12 (nmo ontology) */
 	TRACKER_DB_VERSION_08,      /* Version 0.7.13 (coalesce & writeback) */
 	TRACKER_DB_VERSION_09,      /* Version 0.7.17 (mlo ontology) */
-	TRACKER_DB_VERSION_10       /* Version 0.7.20 (nco im ontology) */
+	TRACKER_DB_VERSION_10,      /* Version 0.7.20 (nco im ontology) */
+	TRACKER_DB_VERSION_11       /* Version 0.7.21 (named graphs/localtime) */
 } TrackerDBVersion;
 
 typedef struct {
@@ -308,6 +309,45 @@ tracker_db_manager_get_flags (void)
 	return old_flags;
 }
 
+static GValue
+function_sparql_string_join (TrackerDBInterface *interface,
+			     gint                argc,
+			     GValue              values[])
+{
+	GValue result = { 0, };
+	GString *str = NULL;
+	const gchar *separator;
+	gint i;
+
+	/* fn:string-join (str1, str2, ..., separator) */
+
+	if (!G_VALUE_HOLDS_STRING (&values[argc-1])) {
+		/* Fail */
+		return result;
+	}
+
+	separator = g_value_get_string (&values[argc-1]);
+
+	for (i = 0;i < argc-1; i++) {
+		if (G_VALUE_HOLDS_STRING (&values[i])) {
+			if (!str) {
+				str = g_string_new (g_value_get_string (&values[i]));
+			} else {
+				g_string_append_printf (str, "%s%s",
+							separator,
+							g_value_get_string (&values[i]));
+			}
+		}
+	}
+
+	g_value_init (&result, G_TYPE_STRING);
+	g_value_set_string (&result, str->str);
+
+	g_string_free (str, TRUE);
+
+	return result;
+}
+
 /* Create a title-type string from the filename for replacing missing ones */
 static GValue
 function_sparql_string_from_filename (TrackerDBInterface *interface,
@@ -353,22 +393,32 @@ function_sparql_cartesian_distance (TrackerDBInterface *interface,
 				    GValue              values[])
 {
 	GValue result = { 0, };
+	gdouble lat1;
+	gdouble lat2;
+	gdouble lon1;
+	gdouble lon2;
+
+	gdouble R;
+	gdouble a;
+	gdouble b;
+	gdouble c;
+	gdouble d;
 
 	if (argc != 4) {
 		g_critical ("Invalid argument count");
 		return result;
 	}
 
-	gdouble lat1 = g_value_get_double (&values[0])*M_PI/180;
-	gdouble lat2 = g_value_get_double (&values[1])*M_PI/180;
-	gdouble lon1 = g_value_get_double (&values[2])*M_PI/180;
-	gdouble lon2 = g_value_get_double (&values[3])*M_PI/180;
+	lat1 = g_value_get_double (&values[0])*M_PI/180;
+	lat2 = g_value_get_double (&values[1])*M_PI/180;
+	lon1 = g_value_get_double (&values[2])*M_PI/180;
+	lon2 = g_value_get_double (&values[3])*M_PI/180;
 
-	gdouble R = 6371000;
-	gdouble a = M_PI/2 - lat1;
-	gdouble b = M_PI/2 - lat2;
-	gdouble c = sqrt(a*a + b*b - 2*a*b*cos(lon2 - lon1));
-	gdouble d = R*c;
+	R = 6371000;
+	a = M_PI/2 - lat1;
+	b = M_PI/2 - lat2;
+	c = sqrt(a*a + b*b - 2*a*b*cos(lon2 - lon1));
+	d = R*c;
 
 	g_value_init (&result, G_TYPE_DOUBLE);
 	g_value_set_double (&result, d);
@@ -382,23 +432,34 @@ function_sparql_haversine_distance (TrackerDBInterface *interface,
 				    GValue              values[])
 {
 	GValue result = { 0, };
+	gdouble lat1;
+	gdouble lat2;
+	gdouble lon1;
+	gdouble lon2;
+
+	gdouble R;
+	gdouble dLat;
+	gdouble dLon;
+	gdouble a;
+	gdouble c;
+	gdouble d;
 
 	if (argc != 4) {
 		g_critical ("Invalid argument count");
 		return result;
 	}
 
-	gdouble lat1 = g_value_get_double (&values[0])*M_PI/180;
-	gdouble lat2 = g_value_get_double (&values[1])*M_PI/180;
-	gdouble lon1 = g_value_get_double (&values[2])*M_PI/180;
-	gdouble lon2 = g_value_get_double (&values[3])*M_PI/180;
+	lat1 = g_value_get_double (&values[0])*M_PI/180;
+	lat2 = g_value_get_double (&values[1])*M_PI/180;
+	lon1 = g_value_get_double (&values[2])*M_PI/180;
+	lon2 = g_value_get_double (&values[3])*M_PI/180;
 
-	gdouble R = 6371000;
-	gdouble dLat = (lat2-lat1);
-	gdouble dLon = (lon2-lon1); 
-	gdouble a = sin(dLat/2) * sin(dLat/2) + cos(lat1) * cos(lat2) *  sin(dLon/2) * sin(dLon/2); 
-	gdouble c = 2 * atan2(sqrt(a), sqrt(1-a)); 
-	gdouble d = R * c;
+	R = 6371000;
+	dLat = (lat2-lat1);
+	dLon = (lon2-lon1);
+	a = sin(dLat/2) * sin(dLat/2) + cos(lat1) * cos(lat2) *  sin(dLon/2) * sin(dLon/2);
+	c = 2 * atan2(sqrt(a), sqrt(1-a));
+	d = R * c;
 
 	g_value_init (&result, G_TYPE_DOUBLE);
 	g_value_set_double (&result, d);
@@ -795,6 +856,10 @@ db_set_params (TrackerDBInterface *iface,
 		                                             "SparqlRegex",
 		                                             function_sparql_regex,
 		                                             3);
+		tracker_db_interface_sqlite_create_function (iface,
+		                                             "SparqlStringJoin",
+		                                             function_sparql_string_join,
+		                                             -1);
 		tracker_db_interface_sqlite_create_function (iface,
 		                                             "SparqlStringFromFilename",
 		                                             function_sparql_string_from_filename,
@@ -1459,7 +1524,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 		}
 	}
 
-	in_use_file = g_open (in_use_filename, 
+	in_use_file = g_open (in_use_filename,
 	                      O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
 	                      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
 	fsync (in_use_file);
