@@ -126,24 +126,24 @@ tracker_marshal_VOID__OBJECT_OBJECT_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,BOOLEAN,UINT,UINT,UINT,UINT (tracker-marshal.list:3) */
+/* VOID:OBJECT,POINTER,UINT,UINT,UINT,UINT (tracker-marshal.list:3) */
 void
-tracker_marshal_VOID__POINTER_BOOLEAN_UINT_UINT_UINT_UINT (GClosure     *closure,
-                                                           GValue       *return_value G_GNUC_UNUSED,
-                                                           guint         n_param_values,
-                                                           const GValue *param_values,
-                                                           gpointer      invocation_hint G_GNUC_UNUSED,
-                                                           gpointer      marshal_data)
+tracker_marshal_VOID__OBJECT_POINTER_UINT_UINT_UINT_UINT (GClosure     *closure,
+                                                          GValue       *return_value G_GNUC_UNUSED,
+                                                          guint         n_param_values,
+                                                          const GValue *param_values,
+                                                          gpointer      invocation_hint G_GNUC_UNUSED,
+                                                          gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_BOOLEAN_UINT_UINT_UINT_UINT) (gpointer     data1,
-                                                                          gpointer     arg_1,
-                                                                          gboolean     arg_2,
-                                                                          guint        arg_3,
-                                                                          guint        arg_4,
-                                                                          guint        arg_5,
-                                                                          guint        arg_6,
-                                                                          gpointer     data2);
-  register GMarshalFunc_VOID__POINTER_BOOLEAN_UINT_UINT_UINT_UINT callback;
+  typedef void (*GMarshalFunc_VOID__OBJECT_POINTER_UINT_UINT_UINT_UINT) (gpointer     data1,
+                                                                         gpointer     arg_1,
+                                                                         gpointer     arg_2,
+                                                                         guint        arg_3,
+                                                                         guint        arg_4,
+                                                                         guint        arg_5,
+                                                                         guint        arg_6,
+                                                                         gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_POINTER_UINT_UINT_UINT_UINT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -159,11 +159,11 @@ tracker_marshal_VOID__POINTER_BOOLEAN_UINT_UINT_UINT_UINT (GClosure     *closure
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__POINTER_BOOLEAN_UINT_UINT_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__OBJECT_POINTER_UINT_UINT_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_pointer (param_values + 1),
-            g_marshal_value_peek_boolean (param_values + 2),
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_pointer (param_values + 2),
             g_marshal_value_peek_uint (param_values + 3),
             g_marshal_value_peek_uint (param_values + 4),
             g_marshal_value_peek_uint (param_values + 5),
@@ -290,7 +290,50 @@ tracker_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* BOOL:OBJECT,OBJECT (tracker-marshal.list:7) */
+/* BOOL:OBJECT,OBJECT,OBJECT (tracker-marshal.list:7) */
+void
+tracker_marshal_BOOLEAN__OBJECT_OBJECT_OBJECT (GClosure     *closure,
+                                               GValue       *return_value G_GNUC_UNUSED,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint G_GNUC_UNUSED,
+                                               gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__OBJECT_OBJECT_OBJECT) (gpointer     data1,
+                                                                  gpointer     arg_1,
+                                                                  gpointer     arg_2,
+                                                                  gpointer     arg_3,
+                                                                  gpointer     data2);
+  register GMarshalFunc_BOOLEAN__OBJECT_OBJECT_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__OBJECT_OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_object (param_values + 1),
+                       g_marshal_value_peek_object (param_values + 2),
+                       g_marshal_value_peek_object (param_values + 3),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
+/* BOOL:OBJECT,OBJECT (tracker-marshal.list:8) */
 void
 tracker_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
                                         GValue       *return_value G_GNUC_UNUSED,
@@ -331,7 +374,7 @@ tracker_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:OBJECT,POINTER (tracker-marshal.list:8) */
+/* BOOL:OBJECT,POINTER (tracker-marshal.list:9) */
 void
 tracker_marshal_BOOLEAN__OBJECT_POINTER (GClosure     *closure,
                                          GValue       *return_value G_GNUC_UNUSED,
@@ -372,7 +415,7 @@ tracker_marshal_BOOLEAN__OBJECT_POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:OBJECT (tracker-marshal.list:9) */
+/* BOOL:OBJECT (tracker-marshal.list:10) */
 void
 tracker_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                  GValue       *return_value G_GNUC_UNUSED,
