@@ -22,13 +22,13 @@ extern void tracker_marshal_VOID__OBJECT_OBJECT_BOOLEAN_BOOLEAN (GClosure     *c
                                                                  gpointer      invocation_hint,
                                                                  gpointer      marshal_data);
 
-/* VOID:POINTER,BOOLEAN,UINT,UINT,UINT,UINT (tracker-marshal.list:3) */
-extern void tracker_marshal_VOID__POINTER_BOOLEAN_UINT_UINT_UINT_UINT (GClosure     *closure,
-                                                                       GValue       *return_value,
-                                                                       guint         n_param_values,
-                                                                       const GValue *param_values,
-                                                                       gpointer      invocation_hint,
-                                                                       gpointer      marshal_data);
+/* VOID:OBJECT,POINTER,UINT,UINT,UINT,UINT (tracker-marshal.list:3) */
+extern void tracker_marshal_VOID__OBJECT_POINTER_UINT_UINT_UINT_UINT (GClosure     *closure,
+                                                                      GValue       *return_value,
+                                                                      guint         n_param_values,
+                                                                      const GValue *param_values,
+                                                                      gpointer      invocation_hint,
+                                                                      gpointer      marshal_data);
 
 /* VOID:DOUBLE,UINT,UINT,UINT,UINT (tracker-marshal.list:4) */
 extern void tracker_marshal_VOID__DOUBLE_UINT_UINT_UINT_UINT (GClosure     *closure,
