@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2008, Nokia
  *
@@ -17,22 +16,22 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  *
-  * Authors: Philip Van Hoof <philip@codeminded.be>
+ * Authors: Philip Van Hoof <philip@codeminded.be>
  */
 
-#ifndef _TRACKER_TOPANALYZER_H_
-#define _TRACKER_TOPANALYZER_H_
+#ifndef __TRACKER_EXTRACT_TOPANALYZER_H__
+#define __TRACKER_EXTRACT_TOPANALYZER_H__
 
 #include <glib.h>
 
 G_BEGIN_DECLS
 
 void tracker_topanalyzer_init     (void);
-void tracker_topanalyzer_extract  (const gchar *uri, 
-                                   TrackerSparqlBuilder *metadata, 
+void tracker_topanalyzer_extract  (const gchar *uri,
+                                   TrackerSparqlBuilder *metadata,
                                    gchar **content_type);
 void tracker_topanalyzer_shutdown (void);
 
 G_END_DECLS
 
-#endif
+#endif /* __TRACKER_EXTRACT_TOPANALYZER_H__ */
