@@ -45,7 +45,6 @@
 #define DEFAULT_THROTTLE                         0        /* 0->20 */
 #define DEFAULT_SCAN_TIMEOUT                     0        /* 0->1000 */
 #define DEFAULT_CACHE_TIMEOUT                    60       /* 0->1000 */
-#define DEFAULT_ENABLE_THUMBNAILS                TRUE
 #define DEFAULT_INDEX_MOUNTED_DIRECTORIES        TRUE
 #define DEFAULT_INDEX_REMOVABLE_DEVICES                  TRUE
 #define DEFAULT_INDEX_ON_BATTERY                 FALSE
@@ -64,7 +63,6 @@ typedef struct {
 
 	/* Indexing */
 	gint      throttle;
-	gboolean  enable_thumbnails;
 	gboolean  index_on_battery;
 	gboolean  index_on_battery_first_time;
 	gboolean  index_mounted_directories;
@@ -101,7 +99,8 @@ static void     config_get_property         (GObject           *object,
 static void     config_finalize             (GObject           *object);
 static void     config_constructed          (GObject           *object);
 static void     config_changed              (TrackerConfigFile *file);
-static void     config_load                 (TrackerConfig     *config);
+static void     config_load                 (TrackerConfig     *config,
+                                             gboolean           use_defaults);
 static gboolean config_save                 (TrackerConfig     *config);
 static void     config_create_with_defaults (TrackerConfig     *config,
                                              GKeyFile          *key_file,
@@ -121,7 +120,6 @@ enum {
 
 	/* Indexing */
 	PROP_THROTTLE,
-	PROP_ENABLE_THUMBNAILS,
 	PROP_INDEX_ON_BATTERY,
 	PROP_INDEX_ON_BATTERY_FIRST_TIME,
 	PROP_INDEX_MOUNTED_DIRECTORIES,
@@ -144,7 +142,6 @@ static ObjectToKeyFile conversions[] = {
 	{ G_TYPE_INT,     "cache-timeout",                    GROUP_MONITORS, "CacheTimeout"              },
 	/* Indexing */
 	{ G_TYPE_INT,     "throttle",                         GROUP_INDEXING, "Throttle"                  },
-	{ G_TYPE_BOOLEAN, "enable-thumbnails",                GROUP_INDEXING, "EnableThumbnails"          },
 	{ G_TYPE_BOOLEAN, "index-on-battery",                 GROUP_INDEXING, "IndexOnBattery"            },
 	{ G_TYPE_BOOLEAN, "index-on-battery-first-time",      GROUP_INDEXING, "IndexOnBatteryFirstTime"   },
 	{ G_TYPE_BOOLEAN, "index-mounted-directories",        GROUP_INDEXING, "IndexMountedDirectories"   },
@@ -229,14 +226,6 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                   20,
 	                                                   DEFAULT_THROTTLE,
 	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	g_object_class_install_property (object_class,
-	                                 PROP_ENABLE_THUMBNAILS,
-	                                 g_param_spec_boolean ("enable-thumbnails",
-	                                                       "Enable thumbnails",
-	                                                       " Set to false to completely disable thumbnail generation",
-	                                                       DEFAULT_ENABLE_THUMBNAILS,
-	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	;
 	g_object_class_install_property (object_class,
 	                                 PROP_INDEX_ON_BATTERY,
 	                                 g_param_spec_boolean ("index-on-battery",
@@ -362,10 +351,6 @@ config_set_property (GObject      *object,
 		tracker_config_set_throttle (TRACKER_CONFIG (object),
 		                             g_value_get_int (value));
 		break;
-	case PROP_ENABLE_THUMBNAILS:
-		tracker_config_set_enable_thumbnails (TRACKER_CONFIG (object),
-		                                      g_value_get_boolean (value));
-		break;
 	case PROP_INDEX_ON_BATTERY:
 		tracker_config_set_index_on_battery (TRACKER_CONFIG (object),
 		                                     g_value_get_boolean (value));
@@ -447,9 +432,6 @@ config_get_property (GObject    *object,
 	case PROP_THROTTLE:
 		g_value_set_int (value, priv->throttle);
 		break;
-	case PROP_ENABLE_THUMBNAILS:
-		g_value_set_boolean (value, priv->enable_thumbnails);
-		break;
 	case PROP_INDEX_ON_BATTERY:
 		g_value_set_boolean (value, priv->index_on_battery);
 		break;
@@ -537,14 +519,14 @@ config_constructed (GObject *object)
 {
 	(G_OBJECT_CLASS (tracker_config_parent_class)->constructed) (object);
 
-	config_load (TRACKER_CONFIG (object));
+	config_load (TRACKER_CONFIG (object), TRUE);
 }
 
 static void
 config_changed (TrackerConfigFile *file)
 {
 	/* Reload config */
-	config_load (TRACKER_CONFIG (file));
+	config_load (TRACKER_CONFIG (file), FALSE);
 }
 
 static void
@@ -824,13 +806,17 @@ config_set_ignored_directory_conveniences (TrackerConfig *config)
 }
 
 static void
-config_load (TrackerConfig *config)
+config_load (TrackerConfig *config,
+             gboolean       use_defaults)
 {
 	TrackerConfigFile *file;
 	gint i;
 
 	file = TRACKER_CONFIG_FILE (config);
-	config_create_with_defaults (config, file->key_file, FALSE);
+
+        if (use_defaults) {
+                config_create_with_defaults (config, file->key_file, FALSE);
+        }
 
 	if (!file->file_exists) {
 		tracker_config_file_save (file);
@@ -863,26 +849,34 @@ config_load (TrackerConfig *config)
 			break;
 
 		case G_TYPE_POINTER: {
-			GSList *dirs, *l;
+			GSList *new_dirs, *old_dirs, *l;
 			gboolean check_for_duplicates = FALSE;
+                        gboolean equal;
 
 			is_directory_list = TRUE;
 
-			tracker_keyfile_object_load_string_list (G_OBJECT (file),
-			                                         conversions[i].property,
-			                                         file->key_file,
-			                                         conversions[i].group,
-			                                         conversions[i].key,
-			                                         is_directory_list);
-
 			if (strcmp (conversions[i].property, "index-recursive-directories") != 0 &&
 			    strcmp (conversions[i].property, "index-single-directories") != 0) {
-				continue;
+                                tracker_keyfile_object_load_string_list (G_OBJECT (file),
+                                                                         conversions[i].property,
+                                                                         file->key_file,
+                                                                         conversions[i].group,
+                                                                         conversions[i].key,
+                                                                         is_directory_list,
+                                                                         NULL);
+                                continue;
 			}
 
-			g_object_get (config, conversions[i].property, &dirs, NULL);
+                        tracker_keyfile_object_load_string_list (G_OBJECT (file),
+                                                                 conversions[i].property,
+                                                                 file->key_file,
+                                                                 conversions[i].group,
+                                                                 conversions[i].key,
+                                                                 is_directory_list,
+                                                                 &new_dirs);
+                        g_object_get (config, conversions[i].property, &old_dirs, NULL);
 
-			for (l = dirs; l; l = l->next) {
+			for (l = new_dirs; l; l = l->next) {
 				const gchar *path_to_use;
 
 				/* Must be a special dir */
@@ -916,12 +910,21 @@ config_load (TrackerConfig *config)
 			if (check_for_duplicates) {
 				GSList *filtered;
 
-				filtered = tracker_path_list_filter_duplicates (dirs, ".");
-				g_object_set (config, conversions[i].property, filtered, NULL);
+				filtered = tracker_path_list_filter_duplicates (new_dirs, ".");
+				g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
+				g_slist_free (new_dirs);
 
-				g_slist_foreach (filtered, (GFunc) g_free, NULL);
-				g_slist_free (filtered);
-			}
+                                new_dirs = filtered;
+                        }
+
+                        equal = tracker_gslist_with_string_data_equal (new_dirs, old_dirs);
+
+                        if (!equal) {
+                                g_object_set (config, conversions[i].property, new_dirs, NULL);
+                        }
+
+                        g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
+                        g_slist_free (new_dirs);
 
 			break;
 		}
@@ -1122,18 +1125,6 @@ tracker_config_get_throttle (TrackerConfig *config)
 	return priv->throttle;
 }
 
-gboolean
-tracker_config_get_enable_thumbnails (TrackerConfig *config)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_ENABLE_THUMBNAILS);
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	return priv->enable_thumbnails;
-}
-
 gboolean
 tracker_config_get_index_on_battery (TrackerConfig *config)
 {
@@ -1358,20 +1349,6 @@ tracker_config_set_throttle (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "throttle");
 }
 
-void
-tracker_config_set_enable_thumbnails (TrackerConfig *config,
-                                      gboolean       value)
-{
-	TrackerConfigPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_CONFIG (config));
-
-	priv = TRACKER_CONFIG_GET_PRIVATE (config);
-
-	priv->enable_thumbnails = value;
-	g_object_notify (G_OBJECT (config), "enable-thumbnails");
-}
-
 void
 tracker_config_set_index_on_battery (TrackerConfig *config,
                                      gboolean       value)
@@ -1452,7 +1429,8 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
                                                 GSList        *roots)
 {
 	TrackerConfigPrivate *priv;
-	GSList               *l;
+	GSList *l;
+        gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1460,6 +1438,8 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
 
 	l = priv->index_recursive_directories;
 
+        equal = tracker_gslist_with_string_data_equal (roots, l);
+
 	if (!roots) {
 		priv->index_recursive_directories = NULL;
 	} else {
@@ -1470,6 +1450,10 @@ tracker_config_set_index_recursive_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+        if (equal) {
+                return;
+        }
+
 	g_object_notify (G_OBJECT (config), "index-recursive-directories");
 }
 
@@ -1478,7 +1462,8 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
                                              GSList        *roots)
 {
 	TrackerConfigPrivate *priv;
-	GSList               *l;
+	GSList *l;
+        gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1486,6 +1471,8 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
 
 	l = priv->index_single_directories;
 
+        equal = tracker_gslist_with_string_data_equal (roots, l);
+
 	if (!roots) {
 		priv->index_single_directories = NULL;
 	} else {
@@ -1496,6 +1483,10 @@ tracker_config_set_index_single_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+        if (equal) {
+                return;
+        }
+
 	g_object_notify (G_OBJECT (config), "index-single-directories");
 }
 
@@ -1504,7 +1495,8 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
                                         GSList        *roots)
 {
 	TrackerConfigPrivate *priv;
-	GSList               *l;
+	GSList *l;
+        gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1512,6 +1504,8 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
 
 	l = priv->ignored_directories;
 
+        equal = tracker_gslist_with_string_data_equal (roots, l);
+
 	if (!roots) {
 		priv->ignored_directories = NULL;
 	} else {
@@ -1522,6 +1516,10 @@ tracker_config_set_ignored_directories (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+        if (equal) {
+                return;
+        }
+
 	/* Re-set up the GPatternSpec list */
 	config_set_ignored_directory_conveniences (config);
 
@@ -1533,7 +1531,8 @@ tracker_config_set_ignored_directories_with_content (TrackerConfig *config,
                                                      GSList        *roots)
 {
 	TrackerConfigPrivate *priv;
-	GSList               *l;
+	GSList *l;
+        gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1541,6 +1540,8 @@ tracker_config_set_ignored_directories_with_content (TrackerConfig *config,
 
 	l = priv->ignored_directories_with_content;
 
+        equal = tracker_gslist_with_string_data_equal (roots, l);
+
 	if (!roots) {
 		priv->ignored_directories_with_content = NULL;
 	} else {
@@ -1551,6 +1552,10 @@ tracker_config_set_ignored_directories_with_content (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+        if (equal) {
+                return;
+        }
+
 	g_object_notify (G_OBJECT (config), "ignored-directories-with-content");
 }
 
@@ -1559,7 +1564,8 @@ tracker_config_set_ignored_files (TrackerConfig *config,
                                   GSList        *files)
 {
 	TrackerConfigPrivate *priv;
-	GSList               *l;
+	GSList *l;
+        gboolean equal;
 
 	g_return_if_fail (TRACKER_IS_CONFIG (config));
 
@@ -1567,6 +1573,8 @@ tracker_config_set_ignored_files (TrackerConfig *config,
 
 	l = priv->ignored_files;
 
+        equal = tracker_gslist_with_string_data_equal (files, l);
+
 	if (!files) {
 		priv->ignored_files = NULL;
 	} else {
@@ -1577,6 +1585,10 @@ tracker_config_set_ignored_files (TrackerConfig *config,
 	g_slist_foreach (l, (GFunc) g_free, NULL);
 	g_slist_free (l);
 
+        if (equal) {
+                return;
+        }
+
 	/* Re-set up the GPatternSpec list */
 	config_set_ignored_file_conveniences (config);
 
