@@ -40,6 +40,8 @@
 #include <libtracker-common/tracker-ontology.h>
 #include <libtracker-common/tracker-file-utils.h>
 
+#include <libtracker-miner/tracker-thumbnailer.h>
+
 #include <libtracker-db/tracker-db-manager.h>
 #include <libtracker-db/tracker-db-dbus.h>
 
@@ -222,6 +224,13 @@ miner_finished_cb (TrackerMinerFS *fs,
 	miner_handle_next ();
 }
 
+static void
+finalize_miner (TrackerMiner *miner)
+{
+	g_object_run_dispose (G_OBJECT (miner));
+	g_object_unref (G_OBJECT (miner));
+}
+
 int
 main (gint argc, gchar *argv[])
 {
@@ -319,7 +328,7 @@ main (gint argc, gchar *argv[])
 
 	miner_files = tracker_miner_files_new (config);
 
-	object = tracker_miner_files_reindex_new (miner_files);
+	object = tracker_miner_files_reindex_new (TRACKER_MINER_FILES (miner_files));
 
 	if (!object) {
 		g_object_unref (miner_files);
@@ -356,6 +365,8 @@ main (gint argc, gchar *argv[])
 	                  G_CALLBACK (miner_finished_cb),
 	                  NULL);
 
+	tracker_thumbnailer_init ();
+
 	miner_handle_next ();
 
 	main_loop = g_main_loop_new (NULL, FALSE);
@@ -366,7 +377,9 @@ main (gint argc, gchar *argv[])
 	g_main_loop_unref (main_loop);
 	g_object_unref (config);
 
-	g_slist_foreach (miners, (GFunc) g_object_unref, NULL);
+	tracker_thumbnailer_shutdown ();
+
+	g_slist_foreach (miners, (GFunc) finalize_miner, NULL);
 	g_slist_free (miners);
 
 	tracker_dbus_shutdown ();
