@@ -28,6 +28,7 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
+#include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-ontology.h>
 #include <libtracker-common/tracker-power.h>
 #include <libtracker-common/tracker-storage.h>
@@ -64,6 +65,9 @@ struct TrackerMinerFilesPrivate {
 
 	GVolumeMonitor *volume_monitor;
 
+        GSList *index_recursive_directories;
+        GSList *index_single_directories;
+
 	guint disk_space_check_id;
 	guint disk_space_pause_cookie;
 
@@ -72,6 +76,7 @@ struct TrackerMinerFilesPrivate {
 	DBusGProxy *extractor_proxy;
 
 	GQuark quark_mount_point_udi;
+	GQuark quark_directory_config_root;
 };
 
 enum {
@@ -84,63 +89,65 @@ enum {
 	PROP_CONFIG
 };
 
-static void     miner_files_set_property      (GObject              *object,
-                                               guint                 param_id,
-                                               const GValue         *value,
-                                               GParamSpec           *pspec);
-static void     miner_files_get_property      (GObject              *object,
-                                               guint                 param_id,
-                                               GValue               *value,
-                                               GParamSpec           *pspec);
-static void     miner_files_finalize          (GObject              *object);
-static void     miner_files_constructed       (GObject              *object);
-
-static void     mount_pre_unmount_cb          (GVolumeMonitor       *volume_monitor,
-                                               GMount               *mount,
-                                               TrackerMinerFiles    *mf);
+static void        miner_files_set_property             (GObject              *object,
+                                                         guint                 param_id,
+                                                         const GValue         *value,
+                                                         GParamSpec           *pspec);
+static void        miner_files_get_property             (GObject              *object,
+                                                         guint                 param_id,
+                                                         GValue               *value,
+                                                         GParamSpec           *pspec);
+static void        miner_files_finalize                 (GObject              *object);
+static void        miner_files_constructed              (GObject              *object);
+static void        mount_pre_unmount_cb                 (GVolumeMonitor       *volume_monitor,
+                                                         GMount               *mount,
+                                                         TrackerMinerFiles    *mf);
 
 #ifdef HAVE_HAL
-static void     mount_point_added_cb          (TrackerStorage       *storage,
-                                               const gchar          *udi,
-                                               const gchar          *mount_point,
-                                               gpointer              user_data);
-static void     mount_point_removed_cb        (TrackerStorage       *storage,
-                                               const gchar          *udi,
-                                               const gchar          *mount_point,
-                                               gpointer              user_data);
-
-static void     check_battery_status          (TrackerMinerFiles    *fs);
-static void     battery_status_cb             (GObject              *object,
-                                               GParamSpec           *pspec,
-                                               gpointer              user_data);
+static void        mount_point_added_cb                 (TrackerStorage       *storage,
+                                                         const gchar          *udi,
+                                                         const gchar          *mount_point,
+                                                         gpointer              user_data);
+static void        mount_point_removed_cb               (TrackerStorage       *storage,
+                                                         const gchar          *udi,
+                                                         const gchar          *mount_point,
+                                                         gpointer              user_data);
+static void        check_battery_status                 (TrackerMinerFiles    *fs);
+static void        battery_status_cb                    (GObject              *object,
+                                                         GParamSpec           *pspec,
+                                                         gpointer              user_data);
 #endif
 
-static void     init_mount_points             (TrackerMinerFiles *miner);
-static void     disk_space_check_start        (TrackerMinerFiles    *mf);
-static void     disk_space_check_stop         (TrackerMinerFiles    *mf);
-static void     low_disk_space_limit_cb       (GObject              *gobject,
-                                               GParamSpec           *arg1,
-                                               gpointer              user_data);
-
-static DBusGProxy * extractor_create_proxy    (void);
-
-static gboolean miner_files_check_file        (TrackerMinerFS       *fs,
-                                               GFile                *file);
-static gboolean miner_files_check_directory   (TrackerMinerFS       *fs,
-                                               GFile                *file);
-static gboolean miner_files_check_directory_contents (TrackerMinerFS       *fs,
-                                                      GFile                *parent,
-                                                      GList                *children);
-static gboolean miner_files_process_file      (TrackerMinerFS       *fs,
-                                               GFile                *file,
-                                               TrackerSparqlBuilder *sparql,
-                                               GCancellable         *cancellable);
-static gboolean miner_files_monitor_directory (TrackerMinerFS       *fs,
-                                               GFile                *file);
-static gboolean miner_files_writeback_file    (TrackerMinerFS       *fs,
-                                               GFile                *file,
-                                               TrackerSparqlBuilder *sparql,
-                                               GCancellable         *cancellable);
+static void        init_mount_points                    (TrackerMinerFiles    *miner);
+static void        disk_space_check_start               (TrackerMinerFiles    *mf);
+static void        disk_space_check_stop                (TrackerMinerFiles    *mf);
+static void        low_disk_space_limit_cb              (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
+static void        index_recursive_directories_cb       (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
+static void        index_single_directories_cb          (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
+static DBusGProxy *extractor_create_proxy               (void);
+static gboolean    miner_files_check_file               (TrackerMinerFS       *fs,
+                                                         GFile                *file);
+static gboolean    miner_files_check_directory          (TrackerMinerFS       *fs,
+                                                         GFile                *file);
+static gboolean    miner_files_check_directory_contents (TrackerMinerFS       *fs,
+                                                         GFile                *parent,
+                                                         GList                *children);
+static gboolean    miner_files_process_file             (TrackerMinerFS       *fs,
+                                                         GFile                *file,
+                                                         TrackerSparqlBuilder *sparql,
+                                                         GCancellable         *cancellable);
+static gboolean    miner_files_monitor_directory        (TrackerMinerFS       *fs,
+                                                         GFile                *file);
+static gboolean    miner_files_ignore_next_update_file  (TrackerMinerFS       *fs,
+                                                         GFile                *file,
+                                                         TrackerSparqlBuilder *sparql,
+                                                         GCancellable         *cancellable);
 
 G_DEFINE_TYPE (TrackerMinerFiles, tracker_miner_files, TRACKER_TYPE_MINER_FS)
 
@@ -160,7 +167,7 @@ tracker_miner_files_class_init (TrackerMinerFilesClass *klass)
 	miner_fs_class->check_directory_contents = miner_files_check_directory_contents;
 	miner_fs_class->monitor_directory = miner_files_monitor_directory;
 	miner_fs_class->process_file = miner_files_process_file;
-	miner_fs_class->writeback_file = miner_files_writeback_file;
+	miner_fs_class->ignore_next_update_file = miner_files_ignore_next_update_file;
 
 	g_object_class_install_property (object_class,
 	                                 PROP_CONFIG,
@@ -212,6 +219,7 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	priv->extractor_proxy = extractor_create_proxy ();
 
 	priv->quark_mount_point_udi = g_quark_from_static_string ("tracker-mount-point-udi");
+	priv->quark_directory_config_root = g_quark_from_static_string ("tracker-directory-config-root");
 
 	init_mount_points (mf);
 }
@@ -275,6 +283,16 @@ miner_files_finalize (GObject *object)
 
 	disk_space_check_stop (TRACKER_MINER_FILES (object));
 
+        if (priv->index_recursive_directories) {
+                g_slist_foreach (priv->index_recursive_directories, (GFunc) g_free, NULL);
+                g_slist_free (priv->index_recursive_directories);
+        }
+
+        if (priv->index_single_directories) {
+                g_slist_foreach (priv->index_single_directories, (GFunc) g_free, NULL);
+                g_slist_free (priv->index_single_directories);
+        }
+
 #ifdef HAVE_HAL
 	g_object_unref (priv->power);
 	g_object_unref (priv->storage);
@@ -319,6 +337,9 @@ miner_files_constructed (GObject *object)
 	/* Fill in directories to inspect */
 	dirs = tracker_config_get_index_single_directories (mf->private->config);
 
+        /* Copy in case of config changes */
+        mf->private->index_single_directories = tracker_gslist_copy_with_string_data (dirs);
+
 	for (; dirs; dirs = dirs->next) {
 		GFile *file;
 
@@ -350,8 +371,13 @@ miner_files_constructed (GObject *object)
 		}
 
 		g_message ("  Adding:'%s'", (gchar*) dirs->data);
+
 		file = g_file_new_for_path (dirs->data);
-		tracker_miner_fs_add_directory (fs, file, FALSE);
+		g_object_set_qdata (G_OBJECT (file),
+		                    mf->private->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
+		tracker_miner_fs_directory_add (fs, file, FALSE);
 		g_object_unref (file);
 	}
 
@@ -359,6 +385,9 @@ miner_files_constructed (GObject *object)
 
 	dirs = tracker_config_get_index_recursive_directories (mf->private->config);
 
+        /* Copy in case of config changes */
+        mf->private->index_recursive_directories = tracker_gslist_copy_with_string_data (dirs);
+
 	for (; dirs; dirs = dirs->next) {
 		GFile *file;
 
@@ -390,8 +419,13 @@ miner_files_constructed (GObject *object)
 		}
 
 		g_message ("  Adding:'%s'", (gchar*) dirs->data);
+
 		file = g_file_new_for_path (dirs->data);
-		tracker_miner_fs_add_directory (fs, file, TRUE);
+		g_object_set_qdata (G_OBJECT (file),
+		                    mf->private->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
+		tracker_miner_fs_directory_add (fs, file, TRUE);
 		g_object_unref (file);
 	}
 
@@ -409,8 +443,12 @@ miner_files_constructed (GObject *object)
 					 (GDestroyNotify) g_free);
 #endif
 
+		g_object_set_qdata (G_OBJECT (file),
+		                    mf->private->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
 		g_message ("  Adding:'%s'", (gchar*) m->data);
-		tracker_miner_fs_add_directory (TRACKER_MINER_FS (mf),
+		tracker_miner_fs_directory_add (TRACKER_MINER_FS (mf),
 		                                file,
 		                                TRUE);
 		g_object_unref (file);
@@ -419,6 +457,12 @@ miner_files_constructed (GObject *object)
 	g_signal_connect (mf->private->config, "notify::low-disk-space-limit",
 	                  G_CALLBACK (low_disk_space_limit_cb),
 	                  mf);
+	g_signal_connect (mf->private->config, "notify::index-recursive-directories",
+	                  G_CALLBACK (index_recursive_directories_cb),
+	                  mf);
+	g_signal_connect (mf->private->config, "notify::index-single-directories",
+	                  G_CALLBACK (index_single_directories_cb),
+	                  mf);
 
 	g_slist_foreach (mounts, (GFunc) g_free, NULL);
 	g_slist_free (mounts);
@@ -732,7 +776,11 @@ mount_point_added_cb (TrackerStorage *storage,
 		                         g_strdup (udi),
 		                         (GDestroyNotify) g_free);
 
-		tracker_miner_fs_add_directory (TRACKER_MINER_FS (user_data),
+		g_object_set_qdata (G_OBJECT (file),
+		                    priv->quark_directory_config_root,
+		                    GINT_TO_POINTER (TRUE));
+
+		tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
 		                                file,
 		                                TRUE);
 		g_object_unref (file);
@@ -832,7 +880,7 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
 	GFile *mount_root;
 
 	mount_root = g_mount_get_root (mount);
-	tracker_miner_fs_remove_directory (TRACKER_MINER_FS (mf), mount_root);
+	tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), mount_root);
 	g_object_unref (mount_root);
 }
 
@@ -943,6 +991,115 @@ low_disk_space_limit_cb (GObject    *gobject,
 	disk_space_check_cb (mf);
 }
 
+static void
+update_directories_from_new_config (TrackerMinerFS *mf,
+                                    GSList         *new_dirs,
+                                    GSList         *old_dirs,
+                                    gboolean        recurse)
+{
+	TrackerMinerFilesPrivate *priv;
+        GSList *sl;
+
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (mf);
+
+        g_message ("Updating %s directories changed from configuration",
+                   recurse ? "recursive" : "single");
+
+        /* First remove all directories removed from the config */
+        for (sl = old_dirs; sl; sl = sl->next) {
+                const gchar *path;
+
+                path = sl->data;
+
+                /* If we are not still in the list, remove the dir */
+                if (!tracker_string_in_gslist (path, new_dirs)) {
+                        GFile *file;
+
+                        g_message ("  Removing directory:'%s'", path);
+
+                        file = g_file_new_for_path (path);
+                        tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), file);
+                        g_object_unref (file);
+                }
+        }
+
+        /* Second add directories which are new */
+        for (sl = new_dirs; sl; sl = sl->next) {
+                const gchar *path;
+
+                path = sl->data;
+
+                /* If we are now in the list, add the dir */
+                if (!tracker_string_in_gslist (path, old_dirs)) {
+                        GFile *file;
+
+                        g_message ("  Adding directory:'%s'", path);
+
+                        file = g_file_new_for_path (path);
+			g_object_set_qdata (G_OBJECT (file),
+			                    priv->quark_directory_config_root,
+			                    GINT_TO_POINTER (TRUE));
+
+			tracker_miner_fs_directory_add (TRACKER_MINER_FS (mf), file, recurse);
+                        g_object_unref (file);
+                }
+        }
+}
+
+static void
+index_recursive_directories_cb (GObject    *gobject,
+                                GParamSpec *arg1,
+                                gpointer    user_data)
+{
+        TrackerMinerFilesPrivate *private;
+        GSList *new_dirs, *old_dirs;
+
+        private = TRACKER_MINER_FILES_GET_PRIVATE (user_data);
+
+        new_dirs = tracker_config_get_index_recursive_directories (private->config);
+        old_dirs = private->index_recursive_directories;
+
+        update_directories_from_new_config (TRACKER_MINER_FS (user_data),
+                                            new_dirs,
+                                            old_dirs,
+                                            TRUE);
+
+        /* Re-set the stored config in case it changes again */
+        if (private->index_recursive_directories) {
+                g_slist_foreach (private->index_recursive_directories, (GFunc) g_free, NULL);
+                g_slist_free (private->index_recursive_directories);
+        }
+
+        private->index_recursive_directories = tracker_gslist_copy_with_string_data (new_dirs);
+}
+
+static void
+index_single_directories_cb (GObject    *gobject,
+                             GParamSpec *arg1,
+                             gpointer    user_data)
+{
+        TrackerMinerFilesPrivate *private;
+        GSList *new_dirs, *old_dirs;
+
+        private = TRACKER_MINER_FILES_GET_PRIVATE (user_data);
+
+        new_dirs = tracker_config_get_index_single_directories (private->config);
+        old_dirs = private->index_single_directories;
+
+        update_directories_from_new_config (TRACKER_MINER_FS (user_data),
+                                            new_dirs,
+                                            old_dirs,
+                                            FALSE);
+
+        /* Re-set the stored config in case it changes again */
+        if (private->index_single_directories) {
+                g_slist_foreach (private->index_single_directories, (GFunc) g_free, NULL);
+                g_slist_free (private->index_single_directories);
+        }
+
+        private->index_single_directories = tracker_gslist_copy_with_string_data (new_dirs);
+}
+
 static gboolean
 miner_files_check_file (TrackerMinerFS *fs,
                         GFile          *file)
@@ -1154,6 +1311,7 @@ miner_files_monitor_directory (TrackerMinerFS *fs,
                                GFile          *file)
 {
 	TrackerMinerFiles *mf;
+	GSList *list;
 
 	mf = TRACKER_MINER_FILES (fs);
 
@@ -1161,12 +1319,47 @@ miner_files_monitor_directory (TrackerMinerFS *fs,
 		return FALSE;
 	}
 
+	/* We don't want child directories inside IndexSingleDirectories
+	 * to have a monitor added.
+	 */
+	for (list = mf->private->index_single_directories; list; list = list->next) {
+		gboolean is_child = FALSE;
+		GFile *dir;
+
+		dir = g_file_new_for_path (list->data);
+		is_child = g_file_has_prefix (file, dir);
+		g_object_unref (dir);
+
+		if (is_child) {
+			return FALSE;
+		}
+	}
+
 	/* Fallback to the check directory routine, since we don't
 	 * monitor anything we don't process.
 	 */
 	return miner_files_check_directory (fs, file);
 }
 
+static const gchar *
+miner_files_get_file_urn (TrackerMinerFiles *miner,
+                          GFile             *file,
+			  gboolean          *is_iri)
+{
+	const gchar *urn;
+
+	urn = tracker_miner_fs_get_urn (TRACKER_MINER_FS (miner), file);
+	*is_iri = TRUE;
+
+	if (!urn) {
+		/* This is a new insertion, use anonymous URNs to store files */
+		urn = "_:file";
+		*is_iri = FALSE;
+	}
+
+	return urn;
+}
+
 static void
 miner_files_add_to_datasource (TrackerMinerFiles    *mf,
                                GFile                *file,
@@ -1175,6 +1368,8 @@ miner_files_add_to_datasource (TrackerMinerFiles    *mf,
 	TrackerMinerFilesPrivate *priv;
 	const gchar *removable_device_udi;
 	gchar *removable_device_urn, *uri;
+	const gchar *urn;
+	gboolean is_iri;
 
 	priv = TRACKER_MINER_FILES_GET_PRIVATE (mf);
 	uri = g_file_get_uri (file);
@@ -1192,7 +1387,14 @@ miner_files_add_to_datasource (TrackerMinerFiles    *mf,
 		removable_device_urn = g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN);
 	}
 
-	tracker_sparql_builder_subject (sparql, "_:file");
+	urn = miner_files_get_file_urn (mf, file, &is_iri);
+
+	if (is_iri) {
+		tracker_sparql_builder_subject_iri (sparql, urn);
+	} else {
+		tracker_sparql_builder_subject (sparql, urn);
+	}
+
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
 
@@ -1252,19 +1454,36 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
                                     GError     *error,
                                     gpointer    user_data)
 {
+	TrackerMinerFilesPrivate *priv;
 	ProcessFileData *data = user_data;
 	const gchar *udi;
 
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (data->miner);
+
 	if (error) {
 		/* Something bad happened, notify about the error */
-		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, error);
+		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
 		process_file_data_free (data);
 		g_error_free (error);
 		return;
 	}
 
 	if (sparql && *sparql) {
-		tracker_sparql_builder_append (data->sparql, "\n");
+		gboolean is_iri;
+		const gchar *urn;
+
+		urn = miner_files_get_file_urn (data->miner, data->file, &is_iri);
+
+		if (is_iri) {
+			gchar *str;
+
+			str = g_strdup_printf ("<%s>", urn);
+			tracker_sparql_builder_append (data->sparql, str);
+			g_free (str);
+		} else {
+			tracker_sparql_builder_append (data->sparql, urn);
+		}
+
 		tracker_sparql_builder_append (data->sparql, sparql);
 	}
 
@@ -1313,7 +1532,7 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 	}
 
 	/* Notify about the success */
-	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, NULL);
+	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, NULL);
 
 	process_file_data_free (data);
 	g_free (preupdate);
@@ -1331,7 +1550,7 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 	                          data->call);
 
 	error = g_error_new_literal (miner_files_error_quark, 0, "Embedded metadata extraction was cancelled");
-	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, error);
+	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
 
 	process_file_data_free (data);
 	g_error_free (error);
@@ -1356,23 +1575,26 @@ process_file_cb (GObject      *object,
                  GAsyncResult *result,
                  gpointer      user_data)
 {
+	TrackerMinerFilesPrivate *priv;
 	TrackerSparqlBuilder *sparql;
 	ProcessFileData *data;
-	const gchar *mime_type;
+	const gchar *mime_type, *urn, *parent_urn;
 	GFileInfo *file_info;
 	guint64 time_;
-	GFile *file, *parent;
-	gchar *uri, *parent_uri;
+	GFile *file;
+	gchar *uri;
 	GError *error = NULL;
+	gboolean is_iri;
 
 	data = user_data;
 	file = G_FILE (object);
 	sparql = data->sparql;
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (data->miner);
 	file_info = g_file_query_info_finish (file, result, &error);
 
 	if (error) {
 		/* Something bad happened, notify about the error */
-		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), file, error);
+		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), file, error);
 		process_file_data_free (data);
 		g_error_free (error);
 		return;
@@ -1380,10 +1602,15 @@ process_file_cb (GObject      *object,
 
 	uri = g_file_get_uri (file);
 	mime_type = g_file_info_get_content_type (file_info);
+	urn = miner_files_get_file_urn (TRACKER_MINER_FILES (data->miner), file, &is_iri);
 
 	tracker_sparql_builder_insert_open (sparql, uri);
 
-	tracker_sparql_builder_subject (sparql, "_:file");
+	if (is_iri) {
+		tracker_sparql_builder_subject_iri (sparql, urn);
+	} else {
+		tracker_sparql_builder_subject (sparql, urn);
+	}
 
 	tracker_sparql_builder_predicate (sparql, "a");
 	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
@@ -1393,13 +1620,11 @@ process_file_cb (GObject      *object,
 		tracker_sparql_builder_object (sparql, "nfo:Folder");
 	}
 
-	parent = g_file_get_parent (file);
-	if (parent) {
-		parent_uri = g_file_get_uri (parent);
+	parent_urn = tracker_miner_fs_get_parent_urn (TRACKER_MINER_FS (data->miner), file);
+
+	if (parent_urn) {
 		tracker_sparql_builder_predicate (sparql, "nfo:belongsToContainer");
-		tracker_sparql_builder_object_iri (sparql, parent_uri);
-		g_free (parent_uri);
-		g_object_unref (parent);
+		tracker_sparql_builder_object_iri (sparql, parent_urn);
 	}
 
 	tracker_sparql_builder_predicate (sparql, "nfo:fileName");
@@ -1418,7 +1643,11 @@ process_file_cb (GObject      *object,
 
 	/* Laying the link between the IE and the DO. We use IE = DO */
 	tracker_sparql_builder_predicate (sparql, "nie:isStoredAs");
-	tracker_sparql_builder_object (sparql, "_:file");
+	if (is_iri) {
+		tracker_sparql_builder_object_iri (sparql, urn);
+	} else {
+		tracker_sparql_builder_object (sparql, urn);
+	}
 
 	/* The URL of the DataObject (because IE = DO, this is correct) */
 	tracker_sparql_builder_predicate (sparql, "nie:url");
@@ -1470,10 +1699,10 @@ miner_files_process_file (TrackerMinerFS       *fs,
 }
 
 static gboolean
-miner_files_writeback_file (TrackerMinerFS       *fs,
-                            GFile                *file,
-                            TrackerSparqlBuilder *sparql,
-                            GCancellable         *cancellable)
+miner_files_ignore_next_update_file (TrackerMinerFS       *fs,
+                                     GFile                *file,
+                                     TrackerSparqlBuilder *sparql,
+                                     GCancellable         *cancellable)
 {
 	const gchar *attrs;
 	const gchar *mime_type;
@@ -1494,7 +1723,7 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 	                               cancellable, &error);
 
 	if (error) {
-		g_warning ("Can't writeback: '%s'", error->message);
+		g_warning ("Can't ignore-next-update: '%s'", error->message);
 		g_clear_error (&error);
 		return FALSE;
 	}
@@ -1502,8 +1731,10 @@ miner_files_writeback_file (TrackerMinerFS       *fs,
 	uri = g_file_get_uri (file);
 	mime_type = g_file_info_get_content_type (file_info);
 
-	/* For writeback we only write a few properties back. These properties
-	 * should NEVER be marked as tracker:writeback in the ontology ! */
+	/* For ignore-next-update we only write a few properties back. These properties
+	 * should NEVER be marked as tracker:writeback in the ontology! (else you break
+	 * the tracker-writeback feature) */
+
 	tracker_sparql_builder_insert_open (sparql, uri);
 
 	tracker_sparql_builder_subject_variable (sparql, "urn");
